#========================================================================
# Conditionaly set the CMake policy
#
# This routine tests if the policy is available and set it to STATE
#   if present.
#
# POLICY_ID = The CMake policy identifier
# STATE     = NEW or OLD
#
#------------------------------------------------------------------------
function(cond_set_policy POLICY_ID STATE)
  if( POLICY ${POLICY_ID} )
    cmake_policy( SET ${POLICY_ID} ${STATE})
  endif( POLICY ${POLICY_ID} )
endfunction(cond_set_policy)
#========================================================================
# Retrieve the swig property
#
# var      = Variable to set with the swig property value
# property = Property to be queried
# target   = SWIG target to query
#
# NOTE:
#   MODULE_OUTPUT_NAME is introduced for NDS use.
#
#------------------------------------------------------------------------
function(get_swig_property var property target)
  get_target_property(
    ${var}
    ${SWIG_MODULE_${target}_REAL_NAME}
    ${property}
  )
  if ( "${${var}}" STREQUAL "${var}-NOTFOUND" )
    if ( ${property} STREQUAL "PREFIX" )
      set( ${var} ${CMAKE_SHARED_MODULE_PREFIX} )
    elseif ( ${property} STREQUAL "SUFFIX" )
      set( ${var} ${CMAKE_SHARED_MODULE_SUFFIX} )
    elseif ( ${property} STREQUAL "OUTPUT_NAME" )
      set( ${var} ${SWIG_MODULE_${target}_REAL_NAME} )
    elseif ( ${property} STREQUAL "MODULE_OUTPUT_NAME" )
      get_swig_property( prefix PREFIX ${target} )
      get_swig_property( suffix SUFFIX ${target} )
      get_swig_property( base OUTPUT_NAME ${target} )
      set( ${var} "${prefix}${base}${suffix}" )
    endif ( ${property} STREQUAL "PREFIX" )
  endif ( "${${var}}" STREQUAL "${var}-NOTFOUND" )
  #----------------------------------------------------------------------
  # Elevate the variable
  #----------------------------------------------------------------------
  message( STATUS "${var} set to ${${var}}" )
  set( ${var} ${${var}} PARENT_SCOPE )
endfunction(get_swig_property )